@@ -51,7 +51,9 @@ l_loonPlot <- function(ggObj, panelIndex, args, plotInfo, numOfSubtitles,
                      nDimStates = nDimStates,
                      showGuides = showGuides,
                      linkingKey = linkingKey,
+                     itemLabel = itemLabel,
                      showLabels = plotInfo$showLabels,
+                     showItemLabels = plotInfo$showItemLabels,
                      loonTitle = loonTitle)
 
   } else if (activeModel == "l_hist" & length(activeGeomLayers) > 0) {
@@ -109,6 +111,7 @@ l_loonPlot <- function(ggObj, panelIndex, args, plotInfo, numOfSubtitles,
                   itemLabel = itemLabel,
                   nDimStates = nDimStates,
                   showLabels = plotInfo$showLabels,
+                  showItemLabels = plotInfo$showItemLabels,
                   xlabel = xlabel,
                   ylabel = ylabel,
                   loonTitle = loonTitle,
@@ -120,6 +123,7 @@ l_loonPlot <- function(ggObj, panelIndex, args, plotInfo, numOfSubtitles,
                  showGuides = showGuides,
                  showScales = showScales,
                  showLabels = plotInfo$showLabels,
+                 showItemLabels = plotInfo$showItemLabels,
                  swapAxes = swapAxes,
                  xlabel = if(is.null(xlabel)) "" else xlabel,
                  ylabel = if(is.null(ylabel)) "" else ylabel,
